@@ -2,6 +2,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "input/line_readers.h"
+
 #include "lib/mlr_arch.h"
 #include "lib/mlrutil.h"
 #include "lib/mlr_globals.h"
@@ -109,6 +111,15 @@ static mapper_setup_t* mapper_lookup_table[] = {
 static int mapper_lookup_table_length = sizeof(mapper_lookup_table) / sizeof(mapper_lookup_table[0]);
 
 // ----------------------------------------------------------------
+static void cli_load_mlrrc_or_die(cli_opts_t* popts);
+static int cli_try_load_mlrrc(cli_opts_t* popts, char* path);
+static int handle_mlrrc_line_1(cli_opts_t* popts, char* line);
+static int handle_mlrrc_line_2(cli_opts_t* popts, char* line);
+static int handle_mlrrc_line_3(cli_opts_t* popts, char* line);
+static int handle_mlrrc_line_4(cli_opts_t* popts, char** argv, int argc);
+
+static int cli_handle_misc_options(char** argv, int argc, int *pargi, cli_opts_t* popts);
+
 static lhmss_t* get_desc_to_chars_map();
 static lhmsll_t* get_default_repeat_ifses();
 static lhmsll_t* get_default_repeat_ipses();
@@ -125,6 +136,7 @@ static void main_usage_examples(FILE* o, char* argv0, char* leader);
 static void list_all_verbs_raw(FILE* o);
 static void list_all_verbs(FILE* o, char* leader);
 static void main_usage_help_options(FILE* o, char* argv0);
+static void main_usage_mlrrc(FILE* o, char* argv0);
 static void main_usage_functions(FILE* o, char* argv0, char* leader);
 static void main_usage_data_format_examples(FILE* o, char* argv0);
 static void main_usage_data_format_options(FILE* o, char* argv0);
@@ -155,13 +167,20 @@ static int lhmsll_get_or_die(lhmsll_t* pmap, char* key);
 cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 	cli_opts_t* popts = mlr_malloc_or_die(sizeof(cli_opts_t));
 
+	int argi = 1;
+
+	// Set defaults for options
 	cli_opts_init(popts);
 
-	int no_input       = FALSE;
-	int have_rand_seed = FALSE;
-	unsigned rand_seed = 0;
+	// Try .mlrrc overrides (then command-line on top of that).
+	// A --norc flag (if provided) must come before all other options.
+	// Or, they can set the environment variable MLRRC="__none__".
+	if (argc >= 2 && streq(argv[1], "--norc")) {
+		argi++;
+	} else {
+		cli_load_mlrrc_or_die(popts);
+	}
 
-	int argi = 1;
 	for (; argi < argc; /* variable increment: 1 or 2 depending on flag */) {
 
 		if (argv[argi][0] != '-') {
@@ -174,60 +193,10 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 			// handled
 		} else if (cli_handle_reader_writer_options(argv, argc, &argi, &popts->reader_opts, &popts->writer_opts)) {
 			// handled
-
-		} else if (streq(argv[argi], "-I")) {
-			popts->do_in_place = TRUE;
-			argi += 1;
-
-		} else if (streq(argv[argi], "-n")) {
-			no_input = TRUE;
-			argi += 1;
-
-		} else if (streq(argv[argi], "--from")) {
-			check_arg_count(argv, argi, argc, 2);
-			slls_append(popts->filenames, argv[argi+1], NO_FREE);
-			argi += 2;
-
-		} else if (streq(argv[argi], "--ofmt")) {
-			check_arg_count(argv, argi, argc, 2);
-			popts->ofmt = argv[argi+1];
-			argi += 2;
-
-		} else if (streq(argv[argi], "--nr-progress-mod")) {
-			check_arg_count(argv, argi, argc, 2);
-			if (sscanf(argv[argi+1], "%lld", &popts->nr_progress_mod) != 1) {
-				fprintf(stderr,
-					"%s: --nr-progress-mod argument must be a positive integer; got \"%s\".\n",
-					MLR_GLOBALS.bargv0, argv[argi+1]);
-				main_usage_short(stderr, MLR_GLOBALS.bargv0);
-				exit(1);
-			}
-			if (popts->nr_progress_mod <= 0) {
-				fprintf(stderr,
-					"%s: --nr-progress-mod argument must be a positive integer; got \"%s\".\n",
-					MLR_GLOBALS.bargv0, argv[argi+1]);
-				main_usage_short(stderr, MLR_GLOBALS.bargv0);
-				exit(1);
-			}
-			argi += 2;
-
-		} else if (streq(argv[argi], "--seed")) {
-			check_arg_count(argv, argi, argc, 2);
-			if (sscanf(argv[argi+1], "0x%x", &rand_seed) == 1) {
-				have_rand_seed = TRUE;
-			} else if (sscanf(argv[argi+1], "%u", &rand_seed) == 1) {
-				have_rand_seed = TRUE;
-			} else {
-				fprintf(stderr,
-					"%s: --seed argument must be a decimal or hexadecimal integer; got \"%s\".\n",
-					MLR_GLOBALS.bargv0, argv[argi+1]);
-				main_usage_short(stderr, MLR_GLOBALS.bargv0);
-				exit(1);
-			}
-			argi += 2;
-
-		//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+		} else if (cli_handle_misc_options(argv, argc, &argi, popts)) {
+			// handled
 		} else {
+			// unhandled
 			usage_unrecognized_verb(MLR_GLOBALS.bargv0, argv[argi]);
 		}
 	}
@@ -272,13 +241,13 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 	popts->original_argv = argv;
 	popts->non_in_place_argv = copy_argv(argv);
 	popts->argc = argc;
-	*ppmapper_list = cli_parse_mappers(popts->non_in_place_argv, &argi, argc, popts, &no_input);
+	*ppmapper_list = cli_parse_mappers(popts->non_in_place_argv, &argi, argc, popts);
 
 	for ( ; argi < argc; argi++) {
 		slls_append(popts->filenames, argv[argi], NO_FREE);
 	}
 
-	if (no_input) {
+	if (popts->no_input) {
 		slls_free(popts->filenames);
 		popts->filenames = NULL;
 	}
@@ -288,8 +257,8 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 		exit(1);
 	}
 
-	if (have_rand_seed) {
-		mtrand_init(rand_seed);
+	if (popts->have_rand_seed) {
+		mtrand_init(popts->rand_seed);
 	} else {
 		mtrand_init_default();
 	}
@@ -300,7 +269,7 @@ cli_opts_t* parse_command_line(int argc, char** argv, sllv_t** ppmapper_list) {
 // ----------------------------------------------------------------
 // Returns a list of mappers, from the starting point in argv given by *pargi. Bumps *pargi to
 // point to remaining post-mapper-setup args, i.e. filenames.
-sllv_t* cli_parse_mappers(char** argv, int* pargi, int argc, cli_opts_t* popts, int* pno_input) {
+sllv_t* cli_parse_mappers(char** argv, int* pargi, int argc, cli_opts_t* popts) {
 	sllv_t* pmapper_list = sllv_alloc();
 	int argi = *pargi;
 
@@ -352,7 +321,7 @@ sllv_t* cli_parse_mappers(char** argv, int* pargi, int argc, cli_opts_t* popts,
 
 		if (pmapper_setup->ignores_input && pmapper_list->length == 0) {
 			// e.g. then-chain starts with seqgen
-			*pno_input = TRUE;
+			popts->no_input = TRUE;
 		}
 
 		sllv_append(pmapper_list, pmapper);
@@ -549,6 +518,10 @@ static void main_usage_long(FILE* o, char* argv0) {
 	main_usage_help_options(o, argv0);
 	fprintf(o, "\n");
 
+	fprintf(o, "Customization via .mlrrc:\n");
+	main_usage_mlrrc(o, argv0);
+	fprintf(o, "\n");
+
 	fprintf(o, "Verbs:\n");
 	list_all_verbs(o, "  ");
 	fprintf(o, "\n");
@@ -676,6 +649,40 @@ static void main_usage_help_options(FILE* o, char* argv0) {
 	fprintf(o, "  -K                           Show a bare listing of keywords by name.\n");
 }
 
+static void main_usage_mlrrc(FILE* o, char* argv0) {
+	fprintf(o, "You can set up personal defaults via a $HOME/.mlrrc and/or ./.mlrrc.\n");
+	fprintf(o, "For example, if you usually process CSV, then you can put \"--csv\" in your .mlrrc file\n");
+	fprintf(o, "and that will be the default input/output format unless otherwise specified on the command line.\n");
+	fprintf(o, "\n");
+	fprintf(o, "The .mlrrc file format is one \"--flag\" or \"--option value\" per line, with the leading \"--\" optional.\n");
+	fprintf(o, "Hash-style comments and blank lines are ignored.\n");
+	fprintf(o, "\n");
+	fprintf(o, "Sample .mlrrc:\n");
+	fprintf(o, "# Input and output formats are CSV by default (unless otherwise specified\n");
+	fprintf(o, "# on the mlr command line):\n");
+	fprintf(o, "csv\n");
+	fprintf(o, "# These are no-ops for CSV, but when I do use JSON output, I want these\n");
+	fprintf(o, "# pretty-printing options to be used:\n");
+	fprintf(o, "jvstack\n");
+	fprintf(o, "jlistwrap\n");
+	fprintf(o, "\n");
+	fprintf(o, "How to specify location of .mlrrc:\n");
+	fprintf(o, "* If $MLRRC is set:\n");
+	fprintf(o, "  o If its value is \"__none__\" then no .mlrrc files are processed.\n");
+	fprintf(o, "  o Otherwise, its value (as a filename) is loaded and processed. If there are syntax\n");
+	fprintf(o, "    errors, they abort mlr with a usage message (as if you had mistyped something on the\n");
+	fprintf(o, "    command line). If the file can't be loaded at all, though, it is silently skipped.\n");
+	fprintf(o, "  o Any .mlrrc in your home directory or current directory is ignored whenever $MLRRC is\n");
+	fprintf(o, "    set in the environment.\n");
+	fprintf(o, "* Otherwise:\n");
+	fprintf(o, "  o If $HOME/.mlrrc exists, it's then processed as above.\n");
+	fprintf(o, "  o If ./.mlrrc exists, it's then also processed as above.\n");
+	fprintf(o, "  (I.e. current-directory .mlrrc defaults are stacked over home-directory .mlrrc defaults.)\n");
+	fprintf(o, "\n");
+	fprintf(o, "See also:\n");
+	fprintf(o, "https://johnkerl.org/miller/doc/customization.html\n");
+}
+
 static void main_usage_functions(FILE* o, char* argv0, char* leader) {
 	fmgr_t* pfmgr = fmgr_alloc();
 	fmgr_list_functions(pfmgr, o, leader);
@@ -1092,8 +1099,172 @@ void cli_opts_init(cli_opts_t* popts) {
 	popts->nr_progress_mod = 0LL;
 
 	popts->do_in_place     = FALSE;
+
+	popts->no_input        = FALSE;
+	popts->have_rand_seed  = FALSE;
+	popts->rand_seed       = 0;
 }
 
+// ----------------------------------------------------------------
+// * If $MLRRC is set, use it and only it.
+// * Otherwise try first $HOME/.mlrrc and then ./.mlrrc but let them
+//   stack: e.g. $HOME/.mlrrc is lots of settings and maybe in one
+//   subdir you want to override just a setting or two.
+static void cli_load_mlrrc_or_die(cli_opts_t* popts) {
+	char* env_mlrrc = getenv("MLRRC");
+	if (env_mlrrc != NULL) {
+		if (streq(env_mlrrc, "__none__")) {
+			return;
+		}
+		if (cli_try_load_mlrrc(popts, env_mlrrc)) {
+			return;
+		}
+	}
+
+	char* env_home = getenv("HOME");
+	if (env_home != NULL) {
+		char* path = mlr_paste_2_strings(env_home, "/.mlrrc");
+		(void)cli_try_load_mlrrc(popts, path);
+		free(path);
+	}
+
+	(void)cli_try_load_mlrrc(popts, "./.mlrrc");
+}
+
+static int cli_try_load_mlrrc(cli_opts_t* popts, char* path) {
+	FILE* fp = fopen(path, "r");
+	if (fp == NULL) {
+		return FALSE;
+	}
+
+	char* line = NULL;
+	size_t linecap = 0;
+	int rc;
+	int lineno = 0;
+
+	while ((rc = getline(&line, &linecap, fp)) != -1) {
+		lineno++;
+		char* line_to_destroy = strdup(line);
+		if (!handle_mlrrc_line_1(popts, line_to_destroy)) {
+			fprintf(stderr, "Parse error at file \"%s\" line %d: %s\n",
+				path, lineno, line);
+			exit(1);
+		}
+		free(line_to_destroy);
+	}
+
+	fclose(fp);
+	if (line != NULL) {
+		free(line);
+	}
+
+	return TRUE;
+}
+
+// Chomps trailing CR, LF, or CR/LF; comment-strips; left-right trims.
+static int handle_mlrrc_line_1(cli_opts_t* popts, char* line) {
+	// chomp
+	size_t len = strlen(line);
+	if (len >= 2 && line[len-2] == '\r' && line[len-1] == '\n') {
+		line[len-2] = 0;
+	} else if (len >= 1 && (line[len-1] == '\r' || line[len-1] == '\n')) {
+		line[len-1] = 0;
+	}
+
+	// comment-strip
+	char* pbang = strstr(line, "#");
+	if (pbang != NULL) {
+		*pbang = 0;
+	}
+
+	// Left-trim
+	char* start = line;
+	while (*start == ' ' || *start == '\t') {
+		start++;
+	}
+
+	// Right-trim
+	len = strlen(start);
+	char* end = &start[len-1];
+	while (end > start && (*end == ' ' || *end == '\t')) {
+		*end = 0;
+		end--;
+	}
+	if (end < start) { // line was whitespace-only
+		return TRUE;
+	} else {
+		return handle_mlrrc_line_2(popts, start);
+	}
+}
+
+// Prepends initial "--" if it's not already there
+static int handle_mlrrc_line_2(cli_opts_t* popts, char* line) {
+	size_t len = strlen(line);
+
+	char* dashed_line = NULL;
+	if (len >= 2 && line[0] != '-' && line[1] != '-') {
+		dashed_line = mlr_paste_2_strings("--", line);
+	} else {
+		dashed_line = strdup(line);
+	}
+
+	int rc = handle_mlrrc_line_3(popts, dashed_line);
+
+	// Do not free these. The command-line parsers can retain pointers into argv strings (rather
+	// than copying), resulting in freed-memory reads later in the data-processing verbs.
+	//
+	// It would be possible to be diligent about making sure all current command-line-parsing
+	// callsites copy strings rather than pointing to them -- but it would be easy to miss some, and
+	// also any future codemods might make the same mistake as well.
+	//
+	// It's safer (and no big leak) to simply leave these parsed mlrrc lines unfreed.
+	//
+	// free(dashed_line);
+	return rc;
+}
+
+// Splits line into argv array
+static int handle_mlrrc_line_3(cli_opts_t* popts, char* line) {
+	char* argv[3];
+	int argc = 0;
+	char* split = strpbrk(line, " \t");
+	if (split == NULL) {
+		argv[0] = line;
+		argv[1] = NULL;
+		argc = 1;
+	} else {
+		*split = 0;
+		char* p = split + 1;
+		while (*p == ' ' || *p == '\t') {
+			p++;
+		}
+		argv[0] = line;
+		argv[1] = p;
+		argv[2] = NULL;
+		argc = 2;
+	}
+	return handle_mlrrc_line_4(popts, argv, argc);
+}
+
+static int handle_mlrrc_line_4(cli_opts_t* popts, char** argv, int argc) {
+	int argi = 0;
+	if (cli_handle_reader_options(argv, argc, &argi, &popts->reader_opts)) {
+		// handled
+	} else if (cli_handle_writer_options(argv, argc, &argi, &popts->writer_opts)) {
+		// handled
+	} else if (cli_handle_reader_writer_options(argv, argc, &argi, &popts->reader_opts, &popts->writer_opts)) {
+		// handled
+	} else if (cli_handle_misc_options(argv, argc, &argi, popts)) {
+		// handled
+	} else {
+		// unhandled
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+// ----------------------------------------------------------------
 void cli_reader_opts_init(cli_reader_opts_t* preader_opts) {
 	preader_opts->ifile_fmt                      = NULL;
 	preader_opts->irs                            = NULL;
@@ -1111,7 +1282,6 @@ void cli_reader_opts_init(cli_reader_opts_t* preader_opts) {
 	preader_opts->comment_handling               = COMMENTS_ARE_DATA;
 	preader_opts->comment_string                 = NULL;
 
-	// xxx temp
 	preader_opts->generator_opts.field_name     = "i";
 	preader_opts->generator_opts.start          = 0LL;
 	preader_opts->generator_opts.stop           = 100LL;
@@ -1401,6 +1571,9 @@ static int handle_terminal_usage(char** argv, int argc, int argi) {
 	} else if (streq(argv[argi], "--usage-help-options")) {
 		main_usage_help_options(stdout, MLR_GLOBALS.bargv0);
 		return TRUE;
+	} else if (streq(argv[argi], "--usage-mlrrc")) {
+		main_usage_mlrrc(stdout, MLR_GLOBALS.bargv0);
+		return TRUE;
 	} else if (streq(argv[argi], "--usage-functions")) {
 		main_usage_functions(stdout, MLR_GLOBALS.bargv0, "");
 		return TRUE;
@@ -1503,10 +1676,10 @@ int cli_handle_reader_options(char** argv, int argc, int *pargi, cli_reader_opts
 		preader_opts->ifile_fmt = argv[argi+1];
 		argi += 2;
 
-	} else if (streq(argv[argi], "--igen")) { // xxx temp
+	} else if (streq(argv[argi], "--igen")) {
 		preader_opts->ifile_fmt = "gen";
 		argi += 1;
-	} else if (streq(argv[argi], "--gen-start")) { // xxx temp
+	} else if (streq(argv[argi], "--gen-start")) {
 		preader_opts->ifile_fmt = "gen";
 		check_arg_count(argv, argi, argc, 2);
 		if (sscanf(argv[argi+1], "%lld", &preader_opts->generator_opts.start) != 1) {
@@ -1514,7 +1687,7 @@ int cli_handle_reader_options(char** argv, int argc, int *pargi, cli_reader_opts
 				MLR_GLOBALS.bargv0, argv[argi+1]);
 		}
 		argi += 2;
-	} else if (streq(argv[argi], "--gen-stop")) { // xxx temp
+	} else if (streq(argv[argi], "--gen-stop")) {
 		preader_opts->ifile_fmt = "gen";
 		check_arg_count(argv, argi, argc, 2);
 		if (sscanf(argv[argi+1], "%lld", &preader_opts->generator_opts.stop) != 1) {
@@ -1522,7 +1695,7 @@ int cli_handle_reader_options(char** argv, int argc, int *pargi, cli_reader_opts
 				MLR_GLOBALS.bargv0, argv[argi+1]);
 		}
 		argi += 2;
-	} else if (streq(argv[argi], "--gen-step")) { // xxx temp
+	} else if (streq(argv[argi], "--gen-step")) {
 		preader_opts->ifile_fmt = "gen";
 		check_arg_count(argv, argi, argc, 2);
 		if (sscanf(argv[argi+1], "%lld", &preader_opts->generator_opts.step) != 1) {
@@ -2209,6 +2382,67 @@ int cli_handle_reader_writer_options(char** argv, int argc, int *pargi,
 	return argi != oargi;
 }
 
+// Returns TRUE if the current flag was handled.
+static int cli_handle_misc_options(char** argv, int argc, int *pargi, cli_opts_t* popts) {
+	int argi = *pargi;
+	int oargi = argi;
+
+	if (streq(argv[argi], "-I")) {
+		popts->do_in_place = TRUE;
+		argi += 1;
+
+	} else if (streq(argv[argi], "-n")) {
+		popts->no_input = TRUE;
+		argi += 1;
+
+	} else if (streq(argv[argi], "--from")) {
+		check_arg_count(argv, argi, argc, 2);
+		slls_append(popts->filenames, argv[argi+1], NO_FREE);
+		argi += 2;
+
+	} else if (streq(argv[argi], "--ofmt")) {
+		check_arg_count(argv, argi, argc, 2);
+		popts->ofmt = argv[argi+1];
+		argi += 2;
+
+	} else if (streq(argv[argi], "--nr-progress-mod")) {
+		check_arg_count(argv, argi, argc, 2);
+		if (sscanf(argv[argi+1], "%lld", &popts->nr_progress_mod) != 1) {
+			fprintf(stderr,
+				"%s: --nr-progress-mod argument must be a positive integer; got \"%s\".\n",
+				MLR_GLOBALS.bargv0, argv[argi+1]);
+			main_usage_short(stderr, MLR_GLOBALS.bargv0);
+			exit(1);
+		}
+		if (popts->nr_progress_mod <= 0) {
+			fprintf(stderr,
+				"%s: --nr-progress-mod argument must be a positive integer; got \"%s\".\n",
+				MLR_GLOBALS.bargv0, argv[argi+1]);
+			main_usage_short(stderr, MLR_GLOBALS.bargv0);
+			exit(1);
+		}
+		argi += 2;
+
+	} else if (streq(argv[argi], "--seed")) {
+		check_arg_count(argv, argi, argc, 2);
+		if (sscanf(argv[argi+1], "0x%x", &popts->rand_seed) == 1) {
+			popts->have_rand_seed = TRUE;
+		} else if (sscanf(argv[argi+1], "%u", &popts->rand_seed) == 1) {
+			popts->have_rand_seed = TRUE;
+		} else {
+			fprintf(stderr,
+				"%s: --seed argument must be a decimal or hexadecimal integer; got \"%s\".\n",
+				MLR_GLOBALS.bargv0, argv[argi+1]);
+			main_usage_short(stderr, MLR_GLOBALS.bargv0);
+			exit(1);
+		}
+		argi += 2;
+
+	}
+	*pargi = argi;
+	return argi != oargi;
+}
+
 // ----------------------------------------------------------------
 static char* lhmss_get_or_die(lhmss_t* pmap, char* key) {
 	char* value = lhmss_get(pmap, key);
