@@ -242,9 +242,6 @@ static inline void dbl_sval_end(json_parser_state_t* pstate, json_value_t* ptop)
 	}
 }
 
-#define LINE_AND_COL \
-	state.cur_line, state.cur_col
-
 static const long
 	FLAG_NEXT             = 1 << 0,
 	FLAG_REPROC           = 1 << 1,
@@ -265,7 +262,8 @@ json_value_t * json_parse(
 	const json_char * json,
 	size_t length,
 	char * error_buf,
-	json_char** ppend_of_item)
+	json_char** ppend_of_item,
+	long long *pline_number) // should be set to 0 by the caller before 1st call
 {
 	json_char error[JSON_ERROR_MAX];
 	const json_char * end;
@@ -303,7 +301,7 @@ json_value_t * json_parse(
 		ptop = proot = 0;
 		flags = FLAG_SEEK_VALUE;
 
-		state.cur_line = 1;
+		state.cur_line = *pline_number + 1;
 
 		for (state.ptr = json ;; ++state.ptr) {
 			json_char* pb = (json_char*)((state.ptr == end) ? NULL : state.ptr);
@@ -312,7 +310,7 @@ json_value_t * json_parse(
 			if (flags & FLAG_IN_STRING) {
 				if (!b) {
 					sprintf(error, "Unexpected EOF in string at line %d column %d.",
-						LINE_AND_COL);
+						state.cur_line, state.cur_col);
 					goto e_failed;
 				}
 
@@ -337,7 +335,7 @@ json_value_t * json_parse(
 								(uc_b4 = hex_value (*++state.ptr)) == 0xFF)
 						  {
 								sprintf(error, "Invalid character value `%c` at line %d column %d.",
-									b, LINE_AND_COL);
+									b, state.cur_line, state.cur_col);
 								goto e_failed;
 						  }
 
@@ -355,7 +353,7 @@ json_value_t * json_parse(
 									 (uc_b4 = hex_value (*++state.ptr)) == 0xFF)
 								{
 									sprintf(error, "Invalid character value `%c` at line %d column %d.",
-										b, LINE_AND_COL);
+										b, state.cur_line, state.cur_col);
 									goto e_failed;
 								}
 
@@ -461,17 +459,20 @@ json_value_t * json_parse(
 				*ppend_of_item = pb + 1;
 				break;
 
+				state.cur_col++;
 				switch (b) {
 					WHITESPACE:
 						continue;
 
 					default:
-						sprintf(error, "Line %d column %d: Trailing text: `%c`", state.cur_line, state.cur_col, b);
+						sprintf(error, "Line %d column %d: Trailing text: `%c`",
+							state.cur_line, state.cur_col, b);
 						goto e_failed;
 				}
 			}
 
 			if (flags & FLAG_SEEK_VALUE) {
+				state.cur_col++;
 				switch (b) {
 					WHITESPACE:
 						continue;
@@ -481,7 +482,7 @@ json_value_t * json_parse(
 							flags = (flags & ~ (FLAG_NEED_COMMA | FLAG_SEEK_VALUE)) | FLAG_NEXT;
 						} else {
 							sprintf (error, "Line %d column %d: Unexpected ]",
-								LINE_AND_COL);
+								state.cur_line, state.cur_col);
 							goto e_failed;
 						}
 
@@ -645,7 +646,7 @@ json_value_t * json_parse(
 									continue;
 								} else {
 									sprintf(error, "Line %d column %d: Unexpected `0x%02x` when seeking value",
-										LINE_AND_COL, (unsigned)b);
+										state.cur_line, state.cur_col, (unsigned)b);
 									goto e_failed;
 								}
 						}
@@ -654,13 +655,15 @@ json_value_t * json_parse(
 				switch (ptop->type) {
 				case JSON_OBJECT:
 
+					state.cur_col++;
 					switch (b) {
 						WHITESPACE:
 							continue;
 
 						case '"':
 							if (flags & FLAG_NEED_COMMA) {
-								sprintf(error, "Line %d column %d: Expected , before \"", LINE_AND_COL);
+								sprintf(error, "Line %d column %d: Expected , before \"",
+									state.cur_line, state.cur_col);
 								goto e_failed;
 							}
 
@@ -682,7 +685,8 @@ json_value_t * json_parse(
 							}
 
 						default:
-							sprintf(error, "Line %d column %d: Unexpected `%c` in object", LINE_AND_COL, b);
+							sprintf(error, "Line %d column %d: Unexpected `%c` in object",
+								state.cur_line, state.cur_col, b);
 							goto e_failed;
 					}
 
@@ -696,7 +700,8 @@ json_value_t * json_parse(
 						if (ptop->type == JSON_INTEGER || flags & FLAG_NUM_E) {
 							if (! (flags & FLAG_NUM_E)) {
 								if (flags & FLAG_NUM_ZERO) {
-									sprintf(error, "Line %d column %d: Unexpected `0` before `%c`", LINE_AND_COL, b);
+									sprintf(error, "Line %d column %d: Unexpected `0` before `%c`",
+										state.cur_line, state.cur_col, b);
 									goto e_failed;
 								}
 
@@ -728,7 +733,8 @@ json_value_t * json_parse(
 						}
 					} else if (b == '.' && ptop->type == JSON_INTEGER) {
 						if (!num_digits) {
-							sprintf (error, "Line %d column %d: Expected digit before `.`", LINE_AND_COL);
+							sprintf (error, "Line %d column %d: Expected digit before `.`",
+								state.cur_line, state.cur_col);
 							goto e_failed;
 						}
 
@@ -744,7 +750,8 @@ json_value_t * json_parse(
 					if (! (flags & FLAG_NUM_E)) {
 						if (ptop->type == JSON_DOUBLE) {
 							if (!num_digits) {
-								sprintf(error, "Line %d column %d: Expected digit after `.`", LINE_AND_COL);
+								sprintf(error, "Line %d column %d: Expected digit after `.`",
+									state.cur_line, state.cur_col);
 								goto e_failed;
 							}
 						}
@@ -766,7 +773,8 @@ json_value_t * json_parse(
 						}
 					} else {
 						if (!num_digits) {
-							sprintf(error, "Line %d column %d: Expected digit after `e`", LINE_AND_COL);
+							sprintf(error, "Line %d column %d: Expected digit after `e`",
+								state.cur_line, state.cur_col);
 							goto e_failed;
 						}
 					}
@@ -826,11 +834,13 @@ json_value_t * json_parse(
 		palloc = proot;
 	}
 
+	*pline_number = state.cur_line;
 	return proot;
 
 e_unknown_value:
 
-	sprintf(error, "Line %d column %d: Unknown value", LINE_AND_COL);
+	sprintf(error, "Line %d column %d: Unknown value",
+		state.cur_line, state.cur_col);
 	goto e_failed;
 
 e_alloc_failure:
@@ -840,7 +850,8 @@ e_alloc_failure:
 
 e_overflow:
 
-	sprintf(error, "Line %d column %d: Too long (caught overflow)", LINE_AND_COL);
+	sprintf(error, "Line %d column %d: Too long (caught overflow)",
+		state.cur_line, state.cur_col);
 	goto e_failed;
 
 e_failed:
@@ -864,6 +875,7 @@ e_failed:
 	if (!state.first_pass)
 		json_free_value(proot);
 
+	*pline_number = state.cur_line;
 	return 0;
 }
 
@@ -872,7 +884,8 @@ json_value_t * json_parse_for_unit_test(
 	json_char** ppend_of_item)
 {
 	json_char error_buf[JSON_ERROR_MAX];
-	return json_parse(json, strlen(json), error_buf, ppend_of_item);
+	long long line_number = 0;
+	return json_parse(json, strlen(json), error_buf, ppend_of_item, &line_number);
 }
 
 // ----------------------------------------------------------------
