@@ -145,7 +145,7 @@ static void mapper_put_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "Please see http://johnkerl.org/miller/doc/reference.html for more information\n");
 	fprintf(o, "including function list. Or \"%s -f\".\n", argv0);
 	fprintf(o, "Please see in particular:\n");
-	fprintf(o, "  http://www.johnkerl.org/miller/doc/reference.html#put\n");
+	fprintf(o, "  http://www.johnkerl.org/miller/doc/reference-verbs.html#put\n");
 }
 
 // ----------------------------------------------------------------
@@ -174,8 +174,10 @@ static void mapper_filter_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "  '\n");
 	fprintf(o, "\n");
 	fprintf(o, "Please see http://johnkerl.org/miller/doc/reference.html for more information\n");
-	fprintf(o, "including function list. Or \"%s -f\". Please also also \"%s grep\" which is\n", argv0, argv0);
+	fprintf(o, "including function list. Or \"%s -f\". Please also see \"%s grep\" which is\n", argv0, argv0);
 	fprintf(o, "useful when you don't yet know which field name(s) you're looking for.\n");
+	fprintf(o, "Please see in particular:\n");
+	fprintf(o, "  http://www.johnkerl.org/miller/doc/reference-verbs.html#filter\n");
 }
 
 static void shared_usage(FILE* o, char* argv0, char* verb) {
@@ -219,6 +221,7 @@ static void shared_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "    %s %s put 'begin {@foo = 97} $column += @foo'.\n", argv0, verb);
 	fprintf(o, "    The value part is subject to type-inferencing as specified by -S/-F.\n");
 	fprintf(o, "    May be specified more than once, e.g. -s name1=value1 -s name2=value2.\n");
+	fprintf(o, "    Note: the value may be an environment variable, e.g. -s sequence=$SEQUENCE\n");
 	fprintf(o, "\n");
 
 	fprintf(o, "Tracing options:\n");
